/** Depot4 (c) J. Lampe 1996-2010
*/
package Dp4;

/** Simple stub for logger implementations. By default, it tries to instantiate 
 * <code>LogStunCommonsFactory</code> and routes all logging to logger created from this factory.
 * If this fails, a dummy logger is instantiated that just does nothing.
 * An own LogStub factory can be set by means of <code>setLogStubFactory<7code>. 
 * 
 * Log levels (severity) are defined by simple integers. 0 as base means "normal" or usually called IINFO.
 * More important messages (warning, error,...) have growing numbers 1,2,...
 * More detailed messages (debug, trace,...) grow in opposite direction -1, -2, ...
 * The contract is that there is no level suppressed even if the logging is mapped onto a scheme with a limited
 * number of levels.
 * 
 * This is a very thin wrapper class to logging. Thus, it avoids the introduction of any direct dependency
 * to an external package.
 * 
 * @author lampe
 * @version 
 */
public abstract class LogStub {
	
	private static LogStubFactory currentLogStubFactory;
	
	public static void setLogStubFactory(LogStubFactory ls) {
		currentLogStubFactory= ls;
	}

	/**
    * Convenience method to return a named logger, without the application
    * having to care about factories.
    * This method is placed here, so the user need not bother about a factory at all.
    *
    * @param name Logical name of the <code>LogStub</code> instance to be
    *  returned (the meaning of this name is only known to the underlying
    *  logging implementation that is being wrapped)
    */
	public static LogStub getLog(String name) {
		if (currentLogStubFactory==null) {
			createDefault();
		}
		return currentLogStubFactory.getLog(name);
	}
	
	/** 
	 * @param severity level of message
	 * @param msg message to log
	 */
	public abstract void log(int severity, String msg);
	
	/**
	 * @param severity
	 * @param msg message to log
	 * @param ex exception to log
	 */
	public abstract void log(int severity, String msg, Throwable ex);
	
	/**
	 * @param severity
	 * @return
	 */
	public abstract boolean isLogEnabled(int severity);
	
	private static void createDefault() {
		try {
			Class cla= Class.forName("Dp4.LogStubCommonsFactory");
			currentLogStubFactory= (LogStubFactory)cla.newInstance();
			return;
		} catch (NoClassDefFoundError ignore) {
		} catch (ClassNotFoundException ignore) {
		} catch (InstantiationException ignore) {
		} catch (IllegalAccessException ignore) {
		}
		currentLogStubFactory= new LogStubFactory(){
			public LogStub getLog(String s) {
				return new DummytLogStub() ;
			}
		};
	}
	
	private static class DummytLogStub extends LogStub{

		/* (non-Javadoc)
		 * @see Dp4.LogStub#getLogStub(java.lang.String)
		 */
		public LogStub getLogStub(String s) {
			return this;
		}

		/* (non-Javadoc)
		 * @see Dp4.LogStub#isLogEnabled(int)
		 */
		public boolean isLogEnabled(int severity) {
			return false;
		}

		/* (non-Javadoc)
		 * @see Dp4.LogStub#log(int, java.lang.String, java.lang.Throwable)
		 */
		public void log(int severity, String msg, Throwable ex) {
			
		}

		/* (non-Javadoc)
		 * @see Dp4.LogStub#log(int, java.lang.String)
		 */
		public void log(int severity, String msg) {
		}
		
	}
}
