/** Depot4 for Java (c) J. Lampe 1997-2003
 ASTBuilder.java

 This is a wrapper for all nonterminal invocations, collecting parsing data.
 It builds an Abstract Syntax Tree (AST) that can be referenced by its root
 node.

 Use:
 ====
  Dp4.Translator translator= new Dp4.Translator("MyTranslator");
  // initialize the ASTBuilder:
  ASTBuilder astBuilder= new ASTBuilder();
  astBuilder.clear();
  NTprocMonT.set(translator, astBuilder);

  // call translator:
  Dp4.trgts tt= Dp4.Tools.translate(translator, Dp4.Files.IFile("example.np"), "program");
  if (Dp4.Tools.isOk(translator)) {
    // get the root of the AST:
    ASTNode rootNode= astBuilder.getASTRoot();
    // process it ...
  }


* @author  Juergen Lampe
* @version 1.9.2
* date:   17.03.2003
*/

package Dp4x;

import java.util.List;
import java.util.Stack;

import Dp4.*;

public class ASTBuilder extends NTprocMonT {

	protected Stack stack= new Stack();  // this is the parsing stack
	protected ASTNode astNode;
	private boolean isNotBPObserving = true;

	public ASTBuilder() {
	}

	/** Initializes the data structures */
	public void clear() {
		stack.clear();
		astNode= new ASTDefaultNode("<root>");
		stack.push(astNode);
		isNotBPObserving = true;
	}

	/** Returns the root node of the AST or null if no AST has been built. */
	public ASTNode getASTRoot(){
		if (!astNode.hasChildren()){
			// this is only due to an error
			return null;
		}
		return (ASTNode)astNode.getChildren().get(0);
	}

//----------------------- parse -----------------------
	/** Overwrite superclass method to handling on invoking ans exiting
	 * parsing method.
	 * @param nonTerm
	 * @param p
	 * @return
	 */
	public trgts parse(NonTermT nonTerm, Prs p) {
		trgts h;
		if (isNotBPObserving) {
			Prs.addBrPtObs(p, new ASTBuilderBP(this));
			isNotBPObserving = false;
		}
		final ASTDefaultNode node = new ASTDefaultNode(nonTerm.name);
		stack.push(node);

		h = super.parse(nonTerm, p);

	  stack.pop();   // must hold ==node
		if (p.A){
			node.setTargets(h);
			((ASTNode)stack.peek()).addChild(node);
		}

		return h;
	}

	int getTopChildCount(){
		ASTDefaultNode node= ((ASTDefaultNode)stack.peek());
		return node.getChildren().size();
	}

	void setTopChildCount(int count){
		ASTDefaultNode node= ((ASTDefaultNode)stack.peek());
		List children = node.getChildren();
		if (count<children.size()){
			children.subList(count, children.size()).clear();
		}
	}
}


/* ------------------------------------------------------- */
/** A branch point observer for building the AST */
class ASTBuilderBP extends BrPtObs {
	private final ASTBuilder astBuilder;
	private int stackPointer = -1, stackSize = 64;
	// this is the stack for the indeterministic parsing, only active branchpoints are saved herein
	private int[][] stack = new int[stackSize][2];

	ASTBuilderBP(ASTBuilder c) {
		astBuilder = c;
	}

	//----------------------------- ND ------------------------
	public void ND(Prs0 P) {
		super.ND(P);
		stackPointer = -1;
	}

	//----------------------------- BT ------------------------
	public void BT(Prs0 P) {
		super.BT(P);
		if (stackPointer>=0) {
			int poi= stack[stackPointer][0];
			while(poi<astBuilder.stack.size()){
				astBuilder.stack.pop();
			}
			astBuilder.setTopChildCount(stack[stackPointer][1]);
		}
	}

	//----------------------------- DB ------------------------
	public void DB(Prs0 P) {
		super.DB(P);
		if (stackPointer>-1) stackPointer--;
	}

	//----------------------------- SB ------------------------
	public void SB(Prs0 P) {
		super.SB(P);
		if (stackPointer>=stackSize-1) {
			stackSize += stackSize;
			stack = resize(stackSize, stack);
		}
		stack[++stackPointer][0] = astBuilder.stack.size();
		stack[stackPointer][1]= astBuilder.getTopChildCount();
	}

	/** Utility method for increasing the integer array */
	public static int[][] resize(int ns, int[][] old) {
		int[][] nw = new int[ns][2];
		System.arraycopy(old, 0, nw, 0, old.length);
		return nw;
	}

}
